#include <ga/GASimpleGA.h>	
#include <ga/GA2DBinStrGenome.h> 
#include <ga/std_stream.h>


float Objective(GAGenome &);  
void initsimulation();
void closesimulation();

int
main(int argc, char **argv)
{
  initsimulation();
  printf("Example 1\n\n");
  printf("This program tries to fill a 2DBinaryStringGenome with\n");
  printf("alternating 1s and 0s using a SimpleGA\n\n"); 

  for(int ii=1; ii<argc; ii++) {
    if(strcmp(argv[ii++],"seed") == 0) {
      GARandomSeed((unsigned int)atoi(argv[ii]));
    }
  }

  int width    = 20;  // sirka genomu
  int height   = 3;   // vyska genomu
  int popsize  = 100; // velkost populacie
  int ngen     = 85;  // pocet generacii
  float pmut   = 0.02;// pravdepodobnost mutacie
  float pcross = 0.7; // pravdepodobnost krizenia

  // vytvorime genom typu 2D binarny string
  GA2DBinaryStringGenome genome(width, height, Objective); 

  // vytvorenie genetickeho algoritmu a priradenie vsetkych poloziek, ktore bude potrebovat
  GASimpleGA ga(genome);
  ga.populationSize(popsize);
  ga.nGenerations(ngen);
  ga.pMutation(pmut);
  ga.pCrossover(pcross);
  GATournamentSelector selector;
  ga.selector(selector);

  // sem budeme vypisovat vysledky, po kazdej generacii vypise najlepsieho jedinca
  ga.scoreFilename("bog.dat");
  ga.flushFrequency(1);	
  printf("initializing...");
  // spustime genetcicky algoritmus
  ga.initialize();
  printf("evolving for %d generations...", ga.nGenerations());
  while(!ga.done()){
    ga.step();
    printf(".");
	// vypiseme, s akymi hodnotami bol spusteny najlepsi jedinec generacie
	FILE *fd = fopen("jedince.txt", "a+");

	GA2DBinaryStringGenome & best = (GA2DBinaryStringGenome &) ga.statistics().bestIndividual();

    int a = 0;
    for(int i=0; i<best.width(); i++){
		a <<= 1;
		a += best.gene(i, 0);
	}

	int b = 0;
    for(int i=0; i<best.width(); i++){
		b <<= 1;
		b += best.gene(i, 1);
	}

	int c = 0;
    for(int i=0; i<best.width(); i++){
		c <<= 1;
		c += best.gene(i, 2);
	}

    fprintf(fd, "%e %e %e\n", (float)(a/1048576.0), (float)(b/1048576.0), (float)(c/1048576.0));
	fclose(fd);
  }
  printf("\n");

  // ukoncime simulaciu
  closesimulation();
  return 0;
}
 



float evaluate(double a, double b, double c);

float
Objective(GAGenome& g) {
	
	GA2DBinaryStringGenome & gg = (GA2DBinaryStringGenome &) g;

	// vytvorime si cislo a z prveho riadku genomu
    int a = 0;
    for(int i=0; i<gg.width(); i++){
		a <<= 1;
		a += gg.gene(i, 0);
	}

	// vytvorime si cislo b z druheho riadku genomu
	int b = 0;
    for(int i=0; i<gg.width(); i++){
		b <<= 1;
		b += gg.gene(i, 1);
	}

	// vytvorime si cislo c z tretieho riadku genomu
	int c = 0;
    for(int i=0; i<gg.width(); i++){
		c <<= 1;
		c += gg.gene(i, 2);
	}

  // pustime fuknciu evaluate, ktora zabezpecuje simulaciu
  return evaluate((a / 1048576.0), (b / 1048576.0), (c / 1048576.0));
}

