#include <ode/ode.h>
#include <drawstuff/drawstuff.h>
#include "texturepath.h"

#ifdef _MSC_VER
#pragma warning(disable:4244 4305) 
#endif


#ifdef dDOUBLE
#define dsDrawBox dsDrawBoxD
#define dsDrawSphere dsDrawSphereD
#define dsDrawCylinder dsDrawCylinderD
#define dsDrawCapsule dsDrawCapsuleD
#endif

#define NUM 10			// pocet kociek, z ktorych sa robot sklada
#define SIDE (0.2)		// velkost kocky
#define MASS (1.0)		// hmotnost kocky

static dWorldID world;
static dSpaceID space;
static dBodyID body[NUM];
static dJointID joint[NUM-1];
static dJointGroupID contactgroup;
static dGeomID box[NUM];
static dReal speed = 0.3; 
static int joints;

static void nearCallback (void *data, dGeomID o1, dGeomID o2)
{
  dBodyID b1 = dGeomGetBody(o1);
  dBodyID b2 = dGeomGetBody(o2);
  if (b1 && b2 && dAreConnected (b1,b2)) return;

  dContact contact;
  contact.surface.mode = 0;
  contact.surface.mu = dInfinity;
  if (dCollide (o1,o2,1,&contact.geom,sizeof(dContactGeom))) {
    dJointID c = dJointCreateContact (world,contactgroup,&contact);
    dJointAttach (c,b1,b2);
  }
}


// zaciatok simulacie
static void start()
{
  dAllocateODEDataForThread(dAllocateMaskAll);

  static float xyz[3] = {2.1640f,-1.3079f,1.7600f};
  static float hpr[3] = {125.5000f,-17.0000f,0.0000f};
  dsSetViewpoint (xyz,hpr);
}

static bool firstShow = true;
static const dReal* r;
static float r1 = 0;
static float d1, d2, d3, rad, vel;
static float x0, y00, z0;

// vytvorenie a pohyb robota
static void createChain(int pause)
{
  if (!pause) {
	// tu pouzijeme premenne, ktore dostaneme z funkcie Objective
    static double angle = 0;
    angle += d1;
	// sily posobiece na robota v x-ovej, y-ovej a z-ovej osi
	dBodyAddForce(body[NUM -1], d2+d3,d2-d3,1.5*(sin(angle)+1.0));


	dSpaceCollide(space, 0, &nearCallback);
	dWorldStep(world, 0.05);

	dJointGroupEmpty(contactgroup);
  }

  dReal sides[3] = {SIDE,SIDE,SIDE};
  dsSetColor (1,1,0);
  dsSetTexture (DS_WOOD);
  for (int i=0; i<NUM; i++)
	{
		if (firstShow){
			if (i == 9)
			{
				// zistime pociatocne suradnice
				r = dBodyGetPosition(body[i]);
				x0 = r[0];
				y00 = r[1];
				z0 = r[2];
			}
		}

		// vykreslenie kocky
		dsDrawBox (dBodyGetPosition(body[i]), dBodyGetRotation(body[i]), sides);
	}
}

static int numberOfSteps = 0;
static float fitness = 0;

// jeden krok simulacie
static void simLoop (int pause)
{
  numberOfSteps++;

  if (numberOfSteps == 0)
  {
	  firstShow = true;
  }

  if (numberOfSteps > 5500)
  {
	  // zistime koncove suradnice robota
	  const dReal* r2 = dBodyGetPosition(body[9]);
	  float x1, y1, z1;
	  x1 = r2[0];
	  y1 = r2[1];
	  z1 = r2[2];

	  // vypocitame fitness -  vzdialenost, ktoru presiel
	  fitness = sqrt((x1-x0)*(x1-x0) + (y1-y00)*(y1-y00) + (z1-z0)*(z1-z0));

	  dsStop();
	  return;
  }

  createChain(pause);

  if (firstShow) 
  {
	  firstShow = false;
  }
}


void initsimulation()
{
  dInitODE2(0);

  world = dWorldCreate();
  space = dHashSpaceCreate (0);
  contactgroup = dJointGroupCreate (0);

  dWorldSetGravity(world, 0, 0, -0.5);
  dWorldSetCFM (world,1e-5);
  dCreatePlane (space,0,0,1,0);
}

void closesimulation()
{

  dCloseODE();
}

float evaluate (double a, double b, double c/*, double v*/)
{
  // ulozime si hodnoty z funkcie Objective
  d1 = a;
  d2 = b;
  d3 = c;
  
  dsFunctions fn;
  fn.version = DS_VERSION;
  fn.start = &start;
  fn.step = &simLoop;
  fn.command = 0;
  fn.stop = 0;
  fn.path_to_textures = DRAWSTUFF_TEXTURE_PATH;

  numberOfSteps = 0;

  world = dWorldCreate();
  space = dHashSpaceCreate (0);
  contactgroup = dJointGroupCreate (0);

  int i;
  dWorldSetGravity(world, 0, 0, -0.5);
  dWorldSetCFM (world,1e-5);
  dCreatePlane (space,0,0,1,0);
  

  for (i=0; i<NUM; i++) {
	body[i] = dBodyCreate (world);
	dReal k = i*SIDE;
	dBodySetPosition (body[i], k,k,k+0.4);
	dMass m;
	m.setBox (1,SIDE,SIDE,SIDE);
    m.adjust (MASS);
	dBodySetMass (body[i], &m);
	dBodySetData (body[i], (void*)(size_t)i);
    box[i] = dCreateBox (space,SIDE,SIDE,SIDE);
	dGeomSetBody(box[i], body[i]);
  }
  for (i=0; i<(NUM-1); i++) {
	joint[i] = dJointCreateBall (world, 0);
	dJointAttach (joint[i], body[i], body[i+1]);
    dReal k = (i+0.5)*SIDE;
	dJointSetBallAnchor (joint[i], k,k,k+0.4);
  }

  // pustime simulaciu
  char *ar[] ={"none"};
  firstShow = true;
  dsSimulationLoop (1,ar,352,288,&fn);

  dJointGroupDestroy (contactgroup);
  dSpaceDestroy (space);
  dWorldDestroy (world);

  // vrati fitness
  return fitness;
}
